@@ -1,5 +1,5 @@
 package Bio::ToolBox::Data::core;
-our $VERSION = '1.35';
+our $VERSION = '1.40';
 
 =head1 NAME
 
@@ -29,7 +29,7 @@ sub new {
 	my $class = shift;
 	
 	# in case someone calls this from an established object
-	if (ref($class) =~ /Bio::ToolBox/) {
+	if (ref($class)) {
 		$class = ref($class);
 	}
 	
@@ -53,6 +53,7 @@ sub new {
 		'path'           => undef,
 		'comments'       => [],
 		'data_table'     => [],
+		'header_line_count' => 0,
 	);
 	
 	# Finished
@@ -331,6 +332,17 @@ sub verify {
 				$error .= " Column 5 not strand values.";
 			}
 		}
+		if ($self->{'number_columns'} == 12) {
+			# bed12 has extra special limitations
+			unless ($self->_column_is_stranded(6,7,9) ) {
+				$bed_check = 0;
+				$error .= " Column 6,7,9 not integers.";
+			}
+			unless ($self->_column_is_comma_integers(10,11) ) {
+				$bed_check = 0;
+				$error .= " Column 10,11 not comma-delimited integers.";
+			}
+		}
 		
 		# peak file format
 		if ($self->{'extension'} and 
@@ -409,6 +421,10 @@ sub verify {
 				$ucsc_check = 0;
 				$error .= " Columns 4,5,6,7,8 not integers.";
 			}
+			unless ($self->_column_is_comma_integers(9,10)) {
+				$ucsc_check = 0;
+				$error .= " Columns 9,10 not comma-delimited integers.";
+			}
 			unless($self->_column_is_stranded(3)) {
 				$ucsc_check = 0;
 				$error .= " Column 3 not strand values.";
@@ -448,6 +464,10 @@ sub verify {
 				$ucsc_check = 0;
 				$error .=  " Columns 3,4,5,6,7 not integers.";
 			}
+			unless ($self->_column_is_comma_integers(8,9)) {
+				$ucsc_check = 0;
+				$error .= " Columns 8,9 not comma-delimited integers.";
+			}
 			unless($self->_column_is_stranded(2)) {
 				$ucsc_check = 0;
 				$error .= " Column 2 not strand values.";
@@ -482,6 +502,10 @@ sub verify {
 				$ucsc_check = 0;
 				$error .= " Columns 4,5,6,7,8 not integers.";
 			}
+			unless ($self->_column_is_comma_integers(9,10)) {
+				$ucsc_check = 0;
+				$error .= " Columns 9,10 not comma-delimited integers.";
+			}
 			unless($self->_column_is_stranded(3)) {
 				$ucsc_check = 0;
 				$error .= " Column 3 not strand values.";
@@ -516,6 +540,10 @@ sub verify {
 				$ucsc_check = 0;
 				$error .= " Columns 3,4,5,6,7 not integers.";
 			}
+			unless ($self->_column_is_comma_integers(8,9)) {
+				$ucsc_check = 0;
+				$error .= " Columns 8,9 not comma-delimited integers.";
+			}
 			unless($self->_column_is_stranded(2)) {
 				$ucsc_check = 0;
 				$error .= " Column 2 not strand values.";
@@ -643,7 +671,7 @@ sub verify {
 		$self->{'bed'} or 
 		$self->{'gff'} or 
 		$self->{'ucsc'} or
-		$self->{'extension'} =~ /sgr/i
+		($self->{'extension'} and $self->{'extension'} =~ /sgr/i)
 	) {
 		$self->{'headers'} = 0;
 	}
@@ -651,9 +679,9 @@ sub verify {
 		$self->{'bed'} == 0 and 
 		$self->{'gff'} == 0 and 
 		$self->{'ucsc'} == 0 and
-		$self->{'extension'} !~ /sgr/i
+		($self->{'extension'} and $self->{'extension'} !~ /sgr/i)
 	) {
-		$self->{'headers'} = 1;
+		$self->{'headers'} = 1 unless $self->{'headers'} == -1;
 	}
 	
 	# if we have made it here, then there were no major structural problems
@@ -677,6 +705,22 @@ sub _column_is_integers {
 	return 1;
 }
 
+# internal method to check if a column is nothing but comma delimited integers
+sub _column_is_comma_integers {
+	my $self = shift;
+	my @index = @_;
+	return 1 if ($self->{last_row} == 0); # can't check if table is empty
+	foreach (@index) {
+		return 0 unless exists $self->{$_};
+	}
+	for my $row (1 .. $self->{last_row}) {
+		for my $i (@index) {
+			return 0 unless ($self->{data_table}->[$row][$i] =~ /^[\d,]+$/);
+		}
+	}
+	return 1;
+}
+
 # internal method to check if a column looks like a strand column
 sub _column_is_stranded {
 	my ($self, $index) = @_;
@@ -697,6 +741,7 @@ sub open_database {
 	my $self = shift;
 	my $force = shift || 0;
 	return unless $self->{db};
+	return if $self->{db} =~ /^Parsed:/; # we don't open parsed annotation files
 	if (exists $self->{db_connection}) {
 		return $self->{db_connection} unless $force;
 	}
@@ -924,6 +969,13 @@ sub number_columns {
 	return $self->{number_columns};
 }
 
+sub last_column {
+	my $self = shift;
+	carp "last_column is a read only method" if @_;
+	return $self->{number_columns} - 1;
+}
+
+
 sub last_row {
 	my $self = shift;
 	carp "last_row is a read only method" if @_;
@@ -986,6 +1038,73 @@ sub delete_comment {
 	}
 }
 
+sub vcf_headers {
+	my $self = shift;
+	return unless $self->vcf;
+	return $self->{vcf_headers} if exists $self->{vcf_headers};
+	my $headers = {};
+	foreach my $comment ($self->comments) {
+		my ($key, $value);
+		if ($comment =~ /^##([\w\-\.]+)=(.+)$/) {
+			$key = $1;
+			$value = $2;
+		}
+		else {
+			# invalid vcf header format!?
+			next;
+		}
+		if ($value !~ /^<.+>$/) {
+			# simple value
+			$headers->{$key} = $value;
+		}
+		else {
+			# process complex values
+			# extract ID with regex which should have
+			my $id = ($value =~ /ID=([\w\-\.:]+)/)[0]; 
+			$headers->{$key}{$id} = $value;
+		}
+	}
+	
+	# store and return
+	$self->{vcf_headers} = $headers;
+	return $headers;
+}
+
+sub rewrite_vcf_headers {
+	my $self = shift;
+	return unless $self->vcf;
+	return unless exists $self->{vcf_headers};
+	my @newComments;
+	
+	# file format always comes first
+	push @newComments, sprintf("##fileformat=%s\n", 
+		$self->{vcf_headers}{fileformat});
+	
+	# common attributes
+	foreach my $key (sort {$a cmp $b} keys %{ $self->{vcf_headers} } ) {
+		next if $key eq 'fileformat';
+		if (ref $self->{vcf_headers}{$key} eq 'HASH') {
+			# we have a complex VCF header with multiple keys
+			# we will rewrite for each ID
+			foreach my $id (sort {$a cmp $b} 
+				keys %{ $self->{vcf_headers}{$key} }
+			) {
+				# to avoid complexity of writing correct formatting
+				push @newComments, sprintf("##%s=%s\n", $key, 
+					$self->{vcf_headers}{$key}{$id} );
+			}
+		}
+		else {
+			# a simple value
+			push @newComments, sprintf("##%s=%s\n", $key, 
+				$self->{vcf_headers}{$key} );
+		}
+	}
+	
+	# replace the headers
+	$self->{comments} = \@newComments;
+}
+
 
 
 #### Column Metadata ####
@@ -1186,11 +1305,24 @@ sub id_column {
 	return $self->{column_indices}{id};
 }
 
+
+
+#### Special Row methods ####
+
+sub get_seqfeature {
+	my ($self, $row) = @_;
+	return unless ($row and $row <= $self->{last_row});
+	return unless exists $self->{SeqFeatureObjects};
+	return $self->{SeqFeatureObjects}->[$row] || undef;
+}
+
+
+
 __END__
 
 =head1 METHODS REFERENCE
 
-For reference only. Please use L<Bio::ToolBox::Data>
+For quick reference only. Please see L<Bio::ToolBox::Data> for implementation.
 
 =over 4
 
@@ -1264,6 +1396,10 @@ Returns or sets the VCF version value in the metadata.
 
 Returns the number of columns in the data table.
 
+=item last_column
+
+Returns the array index of the last column in the data table.
+
 =item last_row
 
 Returns the array index of the last row in the data table.
@@ -1296,6 +1432,14 @@ Adds a string to the list of comments to be included in the metadata.
 
 Deletes the indicated array index from the metadata comments array.
 
+=item vcf_headers
+
+Partially parses VCF metadata header lines into a hash structure.
+
+=item rewrite_vcf_headers
+
+Rewrites the vcf headers back into the metadata comments array.
+
 =item list_columns
 
 Returns an array of the column names
@@ -1359,6 +1503,10 @@ Returns the index of the column that best represents the type.
 Returns the index of the column that represents the Primary_ID 
 column used in databases.
 
+=item get_seqfeature
+
+Returns the stored SeqFeature object for a given row.
+
 =back
 
 =head1 AUTHOR
