@@ -12,7 +12,10 @@ use Bio::ToolBox::utility qw(
 	open_to_read_fh 
 	open_to_write_fh 
 ); 
-use Bio::ToolBox::big_helper qw(wig_to_bigwig_conversion);
+use Bio::ToolBox::big_helper qw(
+	open_wig_to_bigwig_fh 
+	generate_chromosome_file
+);
 eval {
 	# check for bam support
 	require Bio::ToolBox::db_helper::bam;
@@ -31,7 +34,7 @@ use constant {
 	LOG2            => log(2),
 	LOG10           => log(10),
 };
-my $VERSION = '1.34';
+my $VERSION = '1.42';
 	
 	
 
@@ -141,7 +144,7 @@ if ($print_version) {
 ### Check for requirements and set defaults
 # global variables
 my ($use_start, $use_mid, $use_span, $bin, $bedgraph, $callback, $split_callback, 
-	$write_wig, $convertor, $data_ref, $outbase, $half_extend);
+	$write_wig, $convertor, $data_ref, $outbase, $half_extend, $chromo_file);
 check_defaults();
 
 # record start time
@@ -236,7 +239,7 @@ else {
 
 
 ### Finish
-printf " Finished in %.1f min\n", (time - $start_time)/60;
+printf " Finished in %.3f min\n", (time - $start_time)/60;
 
 
 
@@ -379,7 +382,7 @@ sub check_defaults {
 		$splice = 0;
 	}
 	if ($shift and $splice) {
-		die " enabling both shift and splices is not allowed. Pick one.\n";
+		die " enabling both shift and splices is currently not allowed. Pick one.\n";
 	}
 	if ($shift and $paired) {
 		warn " disabling shift with paired reads\n";
@@ -417,10 +420,6 @@ sub check_defaults {
 		else {
 			$correlation_min = 0.25;
 		}
-		if ($strand) {
-			warn " disabling strand with shift enabled\n";
-			$strand = 0;
-		}
 	}
 	
 	# check bin size
@@ -466,10 +465,6 @@ sub check_defaults {
 		$outfile = $infile;
 		$outfile =~ s/\.bam$//;
 	}
-	if (defined $gz) {
-		# overide to false if bigwig is true
-		undef $gz if $bigwig;
-	} 
 	(undef, undef, $outbase) = File::Spec->splitpath($outfile);
 	$outfile =~ s/\.(?:wig|bdg|bedgraph)(?:\.gz)?$//i; # strip extension if present
 	$outbase =~ s/\.(?:wig|bdg|bedgraph)(?:\.gz)?$//i; # splitpath doesn't do extensions
@@ -484,17 +479,13 @@ sub check_defaults {
 	}
 	
 	# impossibilities
-	elsif ($strand and $shift) {
-		# this should not happen, strand is disabled with shift
-		die " programming error!\n";
-	}
 	elsif ($use_start and $paired) {
 		# this should not happen, paired start is changed to paired mid
-		die " programming error!\n";
+		die " start and paired programming error! position $position, start $use_start, mid $use_mid, span $use_span, paired $paired, splice $splice, strand $strand, shift $shift, extension value $ext_value, bin $bin, bin size $bin_size\n";
 	}
 	elsif ($shift and $paired) {
 		# this should not happen, shift is disabled with paired
-		die " programming error!\n";
+		die " shift and paired programming error! position $position, start $use_start, mid $use_mid, span $use_span, paired $paired, splice $splice, strand $strand, shift $shift, extension value $ext_value, bin $bin, bin size $bin_size\n";
 	}
 	
 	# start positions
@@ -508,30 +499,40 @@ sub check_defaults {
 		$callback  = \&record_stranded_start;
 		$split_callback = \&record_split_stranded_start;
 		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
-		print " recording stranded, start positions\n";
+		print " recording stranded start positions\n";
+	}
+	elsif ($use_start and $strand and $shift and !$paired) {
+		$callback  = \&record_stranded_shifted_start;
+		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
+		print " recording stranded, shifted start positions\n";
 	}
 	elsif ($use_start and !$strand and $shift and !$paired) {
 		$callback  = \&record_shifted_start;
 		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
-		print " recording shifted-start positions\n";
+		print " recording shifted start positions\n";
 	}
 	
 	# mid positions
 	elsif ($use_mid and $strand and !$shift and $paired) {
 		$callback = \&record_stranded_paired_mid;
 		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
-		print " recording stranded, mid positions of pairs\n";
+		print " recording stranded mid positions of pairs\n";
 	}
 	elsif ($use_mid and $strand and !$shift and !$paired) {
 		$callback = \&record_stranded_mid;
 		$split_callback = \&record_split_stranded_mid;
 		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
-		print " recording stranded, mid positions\n";
+		print " recording stranded mid positions\n";
 	}
 	elsif ($use_mid and !$strand and $shift and !$paired) {
 		$callback  = \&record_shifted_mid;
 		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
-		print " recording shifted-mid positions\n";
+		print " recording shifted mid positions\n";
+	}
+	elsif ($use_mid and $strand and $shift and !$paired) {
+		$callback  = \&record_stranded_shifted_mid;
+		$write_wig = $bin ? \&write_fixstep : \&write_varstep;
+		print " recording stranded, shifted mid positions\n";
 	}
 	elsif ($use_mid and !$strand and !$shift and $paired) {
 		if ($ext_value) {
@@ -622,9 +623,26 @@ sub check_defaults {
 			print " recording shifted positions spanning alignments\n";
 		}
 	}
+	elsif ($use_span and $strand and $shift and !$paired) {
+		if ($position eq 'extend') {
+			$callback  = \&record_stranded_extended;
+			$write_wig = \&write_bedgraph;
+			print " recording stranded extended alignments\n";
+		}
+		elsif ($ext_value) {
+			$callback  = \&record_stranded_shifted_extended;
+			$write_wig = \&write_bedgraph;
+			print " recording stranded, shifted, extended alignments\n";
+		}
+		else {
+			$callback  = \&record_stranded_shifted_span;
+			$write_wig = \&write_bedgraph;
+			print " recording stranded, shifted positions spanning alignments\n";
+		}
+	}
 	else {
 		# what else is left!?
-		die " programming error!\n";
+		die " programming error! position $position, start $use_start, mid $use_mid, span $use_span, paired $paired, splice $splice, strand $strand, shift $shift, extension value $ext_value, bin $bin, bin size $bin_size\n";
 	}
 	
 	
@@ -671,25 +689,35 @@ sub check_defaults {
 
 ### Open the output file handle 
 sub open_wig_file {
-	my ($name, $track) = @_;
+	my ($name, $do_bw) = @_;
 	
-	# add extension to filename
+	# open a bigWig file handle if requested
+	if ($bigwig and $do_bw) {
+		$name .= '.bw' unless $name =~ /\.bw$/;
+		$chromo_file = generate_chromosome_file($sam);
+		my $fh = open_wig_to_bigwig_fh(
+			file   => $name,
+			chromo => $chromo_file,
+		);
+		if ($fh) {
+			return ($name, $fh);
+		}
+		else {
+			# we couldn't open a wigToBigWig filehandle for some reason
+			# so default to standard wig file
+			# typically a failure of wigToBigWig will bring the entire process
+			# down, so this may not be all that necessary
+			print " unable to open a bigWig file, writing standard wig file\n";
+			$name =~ s/\.bw$//;
+		}
+	}
+	
+	# otherwise we open a text wig file
 	$name .= $bedgraph ? '.bdg' : '.wig';
 	$name .= '.gz', if $gz;
-		
-	
-	# open
 	my $fh = open_to_write_fh($name, $gz) or 
 		die " unable to open output wig file '$name'!\n";
 		
-	# write track line
-	if ($bedgraph) {
-		$fh->print("track type=bedGraph\n") if $track;
-	}
-	else {
-		$fh->print("track type=wiggle_0\n") if $track;
-	}
-	
 	# finished
 	return ($name, $fh);
 }
@@ -1149,39 +1177,11 @@ sub write_model_file {
 
 
 
-### Pre-check alignment lengths for extend value
-sub check_alignment_lengths {
-	
-	# start check alignments from the beginning, do not care about chromosome
-	my $bam = $sam->bam;
-	my @lengths;
-	my $count = 0;
-	while (my $a = $bam->read1) {
-		last if $count == 2000; # go through first 2000, seems reasonable
-		next if $a->unmapped;
-		push @lengths, $a->calend - $a->pos;
-		$count++;
-	}
-	
-	# check if the lengths are all equal
-	if (all_equal(\@lengths)) {
-		return $lengths[0];
-	}
-	else {
-		printf "  %s alignments, mean %.2f +/- %.2f, min %s, max %s\n", 
-			scalar @lengths, mean(@lengths), stddev(@lengths), 
-			min(@lengths), max(@lengths);
-		return 0;
-	}
-}
-
-
-
 ### Collect alignment coverage
 sub process_bam_coverage {
 	# using the low level bam coverage method, not strand specific
 	
-	# open wig file
+	# open wig or bigWig file handle
 	my ($filename, $fh) = open_wig_file($outfile, 1);
 	
 	# determine the dump size
@@ -1198,13 +1198,10 @@ sub process_bam_coverage {
 		process_bam_coverage_on_chromosome($fh, $tid, $dump);
 	}
 	
+	# finish
 	$fh->close;
+	unlink $chromo_file if $chromo_file;
 	print " Wrote file $filename\n";
-	
-	# convert to bigwig if requested
-	if ($bigwig) {
-		convert_to_bigwig($filename);
-	}
 }
 
 
@@ -1221,11 +1218,7 @@ sub parallel_process_bam_coverage {
 	# but we want to keep the coverage array reasonable size, 10000 elements
 	# is plenty. we'll do some math to make it a multiple of bin_size to fit
 	my $dump = $bin_size * int( 10000 / $bin_size); 
-	
-	# we don't want child files to be compressed, takes too much CPU time
-	my $original_gz = $gz;
-	$gz = 0;
-	
+		
 	# prepare ForkManager
 	print " Forking into $cpu children for parallel conversion\n";
 	my $pm = Parallel::ForkManager->new($cpu);
@@ -1242,7 +1235,9 @@ sub parallel_process_bam_coverage {
 		$sam->clone;
 		
 		# then write the chromosome coverage in separate chromosome file
-		my ($filename, $fh) = open_wig_file($outfile . '#' . sprintf("%05d", $tid));
+		# do not write bigWig files, just text wig files
+		my ($filename, $fh) = open_wig_file(
+			$outfile . '#' . sprintf("%05d", $tid), 0);
 		process_bam_coverage_on_chromosome($fh, $tid, $dump);
 		
 		# finished with this chromosome
@@ -1251,11 +1246,10 @@ sub parallel_process_bam_coverage {
 	}
 	$pm->wait_all_children;
 	
-	# merge the children files back into one output file
-	print " merging separate chromosome files\n";
+	# find children
 	my @files = glob "$outfile#*";
 	die "can't find children files!\n" unless (@files);
-	$gz = $original_gz;
+	# open new output wig file, bigWig if requested
 	my ($filename, $fh) = open_wig_file($outfile, 1);
 	while (@files) {
 		my $file = shift @files;
@@ -1264,12 +1258,9 @@ sub parallel_process_bam_coverage {
 		$in->close;
 		unlink $file;
 	}
+	$fh->close;
+	unlink $chromo_file if $chromo_file;
 	print " Wrote file $filename\n";
-	
-	# convert to bigwig if requested
-	if ($bigwig) {
-		convert_to_bigwig($filename);
-	}
 }
 
 
@@ -1320,6 +1311,7 @@ sub process_alignments {
 	
 	# open wig files
 	my ($filename1, $filename2, $fh1, $fh2);
+	
 	if ($strand and !$flip) {
 		($filename1, $fh1) = open_wig_file("$outfile\_f", 1);
 		($filename2, $fh2) = open_wig_file("$outfile\_r", 1);
@@ -1351,13 +1343,9 @@ sub process_alignments {
 	# finished
 	$fh1->close;
 	$fh2->close if $fh2;
+	unlink $chromo_file if $chromo_file;
 	print " Wrote file $filename1\n";
 	print " Wrote file $filename2\n" if $filename2;
-	
-	# convert to bigwig if requested
-	if ($bigwig) {
-		convert_to_bigwig($filename1, $filename2);
-	}
 }
 
 
@@ -1365,10 +1353,6 @@ sub process_alignments {
 ### Process alignments in parallel
 sub parallel_process_alignments {
 	
-	# we don't want child files to be compressed, takes too much CPU time
-	my $original_gz = $gz;
-	$gz = 0;
-	
 	# prepare ForkManager
 	print " Forking into $cpu children for parallel conversion\n";
 	my $pm = Parallel::ForkManager->new($cpu);
@@ -1387,15 +1371,20 @@ sub parallel_process_alignments {
 		# open chromosome wig files
 		my ($filename1, $filename2, $fh1, $fh2);
 		if ($strand and !$flip) {
-			($filename1, $fh1) = open_wig_file($outfile . '_f#' . sprintf("%05d", $tid));
-			($filename2, $fh2) = open_wig_file($outfile . '_r#' . sprintf("%05d", $tid));
+			($filename1, $fh1) = open_wig_file(
+				$outfile . '_f#' . sprintf("%05d", $tid), 0);
+			($filename2, $fh2) = open_wig_file(
+				$outfile . '_r#' . sprintf("%05d", $tid), 0);
 		}
 		elsif ($strand and $flip) {
-			($filename1, $fh1) = open_wig_file($outfile . '_r#' . sprintf("%05d", $tid));
-			($filename2, $fh2) = open_wig_file($outfile . '_f#' . sprintf("%05d", $tid));
+			($filename1, $fh1) = open_wig_file(
+				$outfile . '_r#' . sprintf("%05d", $tid), 0);
+			($filename2, $fh2) = open_wig_file(
+				$outfile . '_f#' . sprintf("%05d", $tid), 0);
 		}
 		else {
-			($filename1, $fh1) = open_wig_file($outfile . '#' . sprintf("%05d", $tid));
+			($filename1, $fh1) = open_wig_file(
+				$outfile . '#' . sprintf("%05d", $tid), 0);
 		}
 	
 		# processing depends on whether we need to split splices or not
@@ -1414,6 +1403,7 @@ sub parallel_process_alignments {
 	$pm->wait_all_children;
 	
 	# merge the children files back into one output file
+	# allow to make bigWig file if requested
 	print " merging separate chromosome files\n";
 	my ($filename1, $filename2);
 	if ($strand) {
@@ -1423,10 +1413,8 @@ sub parallel_process_alignments {
 		my @rfiles = glob "$outfile\_r#*";
 		die "can't find children files!\n" unless (@ffiles and @rfiles);
 		
-		$gz = $original_gz;
+		# combine forward files, bigWig if necessary
 		my $fh;
-		
-		# combine forward files
 		($filename1, $fh) = open_wig_file("$outfile\_f", 1);
 		while (@ffiles) {
 			my $file = shift @ffiles;
@@ -1438,7 +1426,7 @@ sub parallel_process_alignments {
 		$fh->close;
 		print " Wrote file $filename1\n";
 		
-		# combine reverse files
+		# combine reverse files, bigWig if necessary
 		($filename2, $fh) = open_wig_file("$outfile\_r", 1);
 		while (@rfiles) {
 			my $file = shift @rfiles;
@@ -1451,11 +1439,10 @@ sub parallel_process_alignments {
 		print " Wrote file $filename2\n";
 	}
 	else {
-		# single wig file
+		# output single wig file, bigWig if requested
 		
 		my @files = glob "$outfile#*";
 		die "can't find children files!\n" unless (@files);
-		$gz = $original_gz;
 		my $fh;
 		($filename1, $fh) = open_wig_file($outfile, 1);
 		while (@files) {
@@ -1468,11 +1455,7 @@ sub parallel_process_alignments {
 		$fh->close;
 		print " Wrote file $filename1\n";
 	}
-	
-	# convert to bigwig if requested
-	if ($bigwig) {
-		convert_to_bigwig($filename1, $filename2);
-	}
+	unlink $chromo_file if $chromo_file;
 }
 
 
@@ -1673,6 +1656,26 @@ sub record_shifted_start {
 }
 
 
+### Record at shifted start position
+sub record_stranded_shifted_start {
+	my ($a, $data) = @_;
+	
+	# check
+	return if $a->qual < $min_mapq;
+	
+	# shift based on strand, record based on strand
+	if ($a->reversed) {
+		# reverse strand
+		$data->{r}{ $a->calend - $shift_value }++;
+	}
+	else {
+		# forward strand
+		$data->{f}{ $a->pos + 1 + $shift_value }++;
+	}
+	check_data($data);
+}
+
+
 ### Record at start position
 sub record_start {
 	my ($a, $data) = @_;
@@ -1777,6 +1780,29 @@ sub record_shifted_mid {
 }
 
 
+### Record at shifted mid position
+sub record_stranded_shifted_mid {
+	my ($a, $data) = @_;
+	
+	# check
+	return if $a->qual < $min_mapq;
+	
+	# calculate mid position
+	my $pos = int( ($a->pos + 1 + $a->calend) / 2);
+	
+	# shift based on strand, record on forward
+	if ($a->reversed) {
+		# reverse strand
+		$data->{r}{ $pos - $shift_value }++;
+	}
+	else {
+		# forward strand
+		$data->{f}{ $pos + $shift_value }++;
+	}
+	check_data($data);
+}
+
+
 ### Record at mid position for paired end
 sub record_paired_mid {
 	my ($a, $data) = @_;
@@ -2055,6 +2081,27 @@ sub record_extended {
 }
 
 
+### Record stranded extended alignment
+sub record_stranded_extended {
+	my ($a, $data) = @_;
+	
+	# check
+	return if $a->qual < $min_mapq;
+	
+	# start based on strand, record based on strand
+	if ($a->reversed) {
+		# reverse strand
+		# must calculate the start position of the 3 prime extended fragment
+		$data->{r}{ $a->calend - $ext_value } .= "$ext_value,";
+	}
+	else {
+		# forward strand
+		$data->{f}{ $a->pos } .= "$ext_value,";
+	}
+	check_data($data);
+}
+
+
 ### Record shifted span alignment
 sub record_shifted_span {
 	my ($a, $data) = @_;
@@ -2077,6 +2124,28 @@ sub record_shifted_span {
 }
 
 
+### Record stranded shifted span alignment
+sub record_stranded_shifted_span {
+	my ($a, $data) = @_;
+	
+	# check
+	return if $a->qual < $min_mapq;
+	
+	# start based on strand, record based on strand
+	my $length = $a->calend - $a->pos;
+	if ($a->reversed) {
+		# reverse strand
+		# start position of the 3 prime extended fragment
+		$data->{r}{ $a->pos - $shift_value } .= "$length,";
+	}
+	else {
+		# forward strand
+		$data->{f}{ $a->pos + $shift_value} .= "$length,";
+	}
+	check_data($data);
+}
+
+
 sub record_shifted_extended {
 	my ($a, $data) = @_;
 	
@@ -2097,6 +2166,26 @@ sub record_shifted_extended {
 }
 
 
+sub record_stranded_shifted_extended {
+	my ($a, $data) = @_;
+	
+	# check
+	return if $a->qual < $min_mapq;
+	
+	# start based on strand, record based on strand
+	if ($a->reversed) {
+		# reverse strand
+		# start position of the 3 prime extended fragment
+		$data->{r}{ $a->pos - $shift_value } .= "$ext_value,";
+	}
+	else {
+		# forward strand
+		$data->{f}{ $a->pos + $shift_value} .= "$ext_value,";
+	}
+	check_data($data);
+}
+
+
 ### Increment count and check the data size for writing
 sub check_data {
 	my $data = $_[0];
@@ -2348,28 +2437,6 @@ sub all_equal {
 }
 
 
-### Convert to BigWig format
-sub convert_to_bigwig {
-	my @files = @_;
-	
-	foreach my $file (@files) {
-		next unless defined $file;
-		my $bw_file = wig_to_bigwig_conversion(
-			'wig'       => $file,
-			'db'        => $sam,
-			'bwapppath' => $bwapp,
-		);
-		if ($bw_file) {
-			print " Converted to $bw_file\n";
-			unlink $file;
-		}
-		else {
-			print " BigWig conversion failed! see standard error for details\n";
-		}
-	}
-}
-
-
 sub _write_array_to_bedgraph {
 	my ($data, $final, $max_pos, $buffer, $offset, $fh, $special) = @_;
 	
@@ -2498,7 +2565,7 @@ bam2wig.pl [--options...] <filename.bam>
   Output options:
   --out <filename> 
   --bw
-  --bwapp </path/to/wigToBigWig or /path/to/bedGraphToBigWig>
+  --bwapp /path/to/wigToBigWig
   --gz
   
   General options:
@@ -2694,12 +2761,12 @@ input file.
 Specify whether or not the wig file should be further converted into 
 an indexed, compressed, binary BigWig file. The default is false.
 
-=item --bwapp < /path/to/wigToBigWig or /path/to/bedGraphToBigWig >
+=item --bwapp /path/to/wigToBigWig
 
-Specify the full path to Jim Kent's bigWig conversion utility. Two 
-different utilities may be used, bedGraphToBigWig or wigToBigWig, 
-depending on the format of the wig file generated. The application 
-paths may be set in the biotoolbox.cfg file.
+Optionally specify the full path to the UCSC wigToBigWig conversion 
+utility. The application path may be set in the .biotoolbox.cfg file 
+or found in the default executable path, which makes this option 
+unnecessary.
 
 =item --gz
 
