@@ -27,7 +27,7 @@ use constant DATASET_HASH_LIMIT => 5001;
 		# region, and a hash returned with potentially a score for each basepair. 
 		# This may become unwieldy for very large regions, which may be better 
 		# served by separate database queries for each window.
-my $VERSION = '1.36';
+my $VERSION = '1.40';
 
 print "\n A script to collect windowed data flanking a relative position of a feature\n\n";
   
@@ -131,7 +131,7 @@ my $start_time = time;
 ## Generate or load the input dataset
 my $Data;
 if ($infile) {
-	$Data = Bio::ToolBox::Data->new(file => $infile) or 
+	$Data = Bio::ToolBox::Data->new(file => $infile, parse => 1) or 
 		die " unable to load input file '$infile'\n";
 	printf " Loaded %s features from $infile.\n", format_with_commas( $Data->last_row );
 	
@@ -942,7 +942,7 @@ sub collect_long_data_window_scores {
 sub go_interpolate_values {
 	
 	# determine counts
-	my $lastwindow = $Data->number_columns - 1; 
+	my $lastwindow = $Data->last_column; 
 		# lastwindow is the index of the last column
 	# walk through each data line and then each window
 	my $stream = $Data->row_stream;
@@ -1003,7 +1003,7 @@ A script to collect data in bins around a relative position.
 get_relative_data.pl --in <in_filename> --out <out_filename> [--options]
   
   Options for existing files:
-  --in <filename> 
+  --in <filename>                  (txt bed gff gtf refFlat ucsc)
   
   Options for new files:
   --db <name|file>
@@ -1035,7 +1035,7 @@ get_relative_data.pl --in <in_filename> --out <out_filename> [--options]
   --gz
   --cpu <integer>                                           (2)
   --version
-  --help
+  --help                              show extended documentation
 
 =head1 OPTIONS
 
@@ -1046,12 +1046,12 @@ The command line flags and descriptions:
 =item --in <filename>
 
 Specify an input file containing either a list of database features or 
-genomic coordinates for which to collect data. The file should be a 
-tab-delimited text file, one row per feature, with columns representing 
-feature identifiers, attributes, coordinates, and/or data values. The 
-first row should be column headers. Bed files are acceptable, as are 
-text files generated by other B<BioToolBox> scripts. Files may be 
-gzipped compressed.
+genomic coordinates for which to collect data. Any tab-delimited text 
+file with recognizable headers is supported. Gene annotation file 
+formats are also supported, including bed, gtf, gff3, refFlat, and 
+UCSC native formats such as gene prediction tables are all supported. 
+Gene annotation files will be parsed as sequence features. 
+Files may be gzipped compressed.
 
 =item --out <filename>
 
